@@ -33,10 +33,9 @@
 #include "http_log.h"
 #include "http_protocol.h"
 
-DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status,
-                                      int error_id, const char *desc)
+DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status, int error_id,
+                                      apr_status_t aprerr, const char *desc)
 {
-    int save_errno = errno;
     dav_error *err = apr_pcalloc(p, sizeof(*err));
 
     /* DBG3("dav_new_error: %d %d %s", status, error_id, desc ? desc : "(no desc)"); */
@@ -44,17 +43,18 @@ DAV_DECLARE(dav_error*) dav_new_error(apr_pool_t *p, int status,
     err->status = status;
     err->error_id = error_id;
     err->desc = desc;
-    err->save_errno = save_errno;
+    err->aprerr = aprerr;
 
     return err;
 }
 
 DAV_DECLARE(dav_error*) dav_new_error_tag(apr_pool_t *p, int status,
-                                          int error_id, const char *desc,
+                                          int error_id, apr_status_t aprerr,
+                                          const char *desc,
                                           const char *namespace,
                                           const char *tagname)
 {
-    dav_error *err = dav_new_error(p, status, error_id, desc);
+    dav_error *err = dav_new_error(p, status, error_id, aprerr, desc);
 
     err->tagname = tagname;
     err->namespace = namespace;
@@ -372,10 +372,8 @@ DAV_DECLARE(const char *) dav_xml_get_cdata(const apr_xml_elem *elem, apr_pool_t
 
     if (strip_white) {
         /* trim leading whitespace */
-        while (apr_isspace(*cdata)) {     /* assume: return false for '\0' */
+        while (apr_isspace(*cdata))     /* assume: return false for '\0' */
             ++cdata;
-            --len;
-        }
 
         /* trim trailing whitespace */
         while (len-- > 0 && apr_isspace(cdata[len]))
@@ -601,6 +599,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
     const char *state_token;
     const char *uri = NULL;        /* scope of current production; NULL=no-tag */
     apr_size_t uri_len = 0;
+    apr_status_t rv;
     dav_if_header *ih = NULL;
     apr_uri_t parsed_uri;
     const dav_hooks_locks *locks_hooks = DAV_GET_HOOKS_LOCKS(r);
@@ -619,36 +618,26 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
             if (list_type == no_tagged
                 || ((uri = dav_fetch_next_token(&str, '>')) == NULL)) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_TAGGED,
+                                     DAV_ERR_IF_TAGGED, 0,
                                      "Invalid If-header: unclosed \"<\" or "
                                      "unexpected tagged-list production.");
             }
 
             /* 2518 specifies this must be an absolute URI; just take the
              * relative part for later comparison against r->uri */
-            if (apr_uri_parse(r->pool, uri, &parsed_uri) != APR_SUCCESS
+            if ((rv = apr_uri_parse(r->pool, uri, &parsed_uri)) != APR_SUCCESS
                 || !parsed_uri.path) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_TAGGED,
+                                     DAV_ERR_IF_TAGGED, rv,
                                      "Invalid URI in tagged If-header.");
             }
             /* note that parsed_uri.path is allocated; we can trash it */
 
             /* clean up the URI a bit */
             ap_getparents(parsed_uri.path);
-
-            /* the resources we will compare to have unencoded paths */
-            if (ap_unescape_url(parsed_uri.path) != OK) {
-                return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_TAGGED,
-                                     "Invalid percent encoded URI in "
-                                     "tagged If-header.");
-            }
-
             uri_len = strlen(parsed_uri.path);
-            if (uri_len > 1 && parsed_uri.path[uri_len - 1] == '/') {
+            if (uri_len > 1 && parsed_uri.path[uri_len - 1] == '/')
                 parsed_uri.path[--uri_len] = '\0';
-            }
 
             uri = parsed_uri.path;
             list_type = tagged;
@@ -663,14 +652,14 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
             if ((list = dav_fetch_next_token(&str, ')')) == NULL) {
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_UNCLOSED_PAREN,
+                                     DAV_ERR_IF_UNCLOSED_PAREN, 0,
                                      "Invalid If-header: unclosed \"(\".");
             }
 
             if ((ih = dav_add_if_resource(r->pool, ih, uri, uri_len)) == NULL) {
                 /* ### dav_add_if_resource() should return an error for us! */
                 return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                     DAV_ERR_IF_PARSE,
+                                     DAV_ERR_IF_PARSE, 0,
                                      "Internal server error parsing \"If:\" "
                                      "header.");
             }
@@ -685,7 +674,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if ((state_token = dav_fetch_next_token(&list, '>')) == NULL) {
                         /* ### add a description to this error */
                         return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                             DAV_ERR_IF_PARSE, NULL);
+                                             DAV_ERR_IF_PARSE, 0, NULL);
                     }
 
                     if ((err = dav_add_if_state(r->pool, ih, state_token, dav_if_opaquelock,
@@ -700,7 +689,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if ((state_token = dav_fetch_next_token(&list, ']')) == NULL) {
                         /* ### add a description to this error */
                         return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                             DAV_ERR_IF_PARSE, NULL);
+                                             DAV_ERR_IF_PARSE, 0, NULL);
                     }
 
                     if ((err = dav_add_if_state(r->pool, ih, state_token, dav_if_etag,
@@ -715,7 +704,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
                     if (list[1] == 'o' && list[2] == 't') {
                         if (condition != DAV_IF_COND_NORMAL) {
                             return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                                 DAV_ERR_IF_MULTIPLE_NOT,
+                                                 DAV_ERR_IF_MULTIPLE_NOT, 0,
                                                  "Invalid \"If:\" header: "
                                                  "Multiple \"not\" entries "
                                                  "for the same state.");
@@ -731,7 +720,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
                 default:
                     return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                         DAV_ERR_IF_UNK_CHAR,
+                                         DAV_ERR_IF_UNK_CHAR, 0,
                                          apr_psprintf(r->pool,
                                                      "Invalid \"If:\" "
                                                      "header: Unexpected "
@@ -750,7 +739,7 @@ static dav_error * dav_process_if_header(request_rec *r, dav_if_header **p_ih)
 
         default:
             return dav_new_error(r->pool, HTTP_BAD_REQUEST,
-                                 DAV_ERR_IF_UNK_CHAR,
+                                 DAV_ERR_IF_UNK_CHAR, 0,
                                  apr_psprintf(r->pool,
                                              "Invalid \"If:\" header: "
                                              "Unexpected character "
@@ -896,7 +885,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     */
     if (flags & DAV_LOCKSCOPE_EXCLUSIVE) {
         if (lock_list != NULL) {
-            return dav_new_error(p, HTTP_LOCKED, 0,
+            return dav_new_error(p, HTTP_LOCKED, 0, 0,
                                  "Existing lock(s) on the requested resource "
                                  "prevent an exclusive lock.");
         }
@@ -914,7 +903,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         */
         for (lock = lock_list; lock != NULL; lock = lock->next) {
             if (lock->scope == DAV_LOCKSCOPE_EXCLUSIVE) {
-                return dav_new_error(p, HTTP_LOCKED, 0,
+                return dav_new_error(p, HTTP_LOCKED, 0, 0,
                                      "The requested resource is already "
                                      "locked exclusively.");
             }
@@ -931,16 +920,13 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         /*
         ** For methods other than LOCK:
         **
-        ** If we have no locks or if the resource is not being modified
-        ** (per RFC 4918 the lock token is not required on resources
-        ** we are not changing), then <seen_locktoken> can be set to true --
+        ** If we have no locks, then <seen_locktoken> can be set to true --
         ** pretending that we've already met the requirement of seeing one
         ** of the resource's locks in the If: header.
         **
         ** Otherwise, it must be cleared and we'll look for one.
         */
-        seen_locktoken = (lock_list == NULL
-                          || flags & DAV_VALIDATE_NO_MODIFY);
+        seen_locktoken = (lock_list == NULL);
     }
 
     /*
@@ -956,7 +942,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         if (seen_locktoken)
             return NULL;
 
-        return dav_new_error(p, HTTP_LOCKED, 0,
+        return dav_new_error(p, HTTP_LOCKED, 0, 0,
                              "This resource is locked and an \"If:\" header "
                              "was not supplied to allow access to the "
                              "resource.");
@@ -980,7 +966,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     if (lock_list == NULL && if_header->dummy_header) {
         if (flags & DAV_VALIDATE_IS_PARENT)
             return NULL;
-        return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+        return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                              "The locktoken specified in the \"Lock-Token:\" "
                              "header is invalid because this resource has no "
                              "outstanding locks.");
@@ -1186,7 +1172,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
                                             "\" submitted a locktoken created "
                                             "by user \"",
                                             lock->auth_user, "\".", NULL);
-                        return dav_new_error(p, HTTP_FORBIDDEN, 0, errmsg);
+                        return dav_new_error(p, HTTP_FORBIDDEN, 0, 0, errmsg);
                     }
 
                     /*
@@ -1306,7 +1292,7 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
                 return NULL;
             }
 
-            return dav_new_error(p, HTTP_LOCKED, 0 /* error_id */,
+            return dav_new_error(p, HTTP_LOCKED, 0 /* error_id */, 0,
                                  "This resource is locked and the \"If:\" "
                                  "header did not specify one of the "
                                  "locktokens for this resource's lock(s).");
@@ -1320,19 +1306,19 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
         ** bad Lock-Token first. That is considered a 400 (Bad Request).
         */
         if (if_header->dummy_header) {
-            return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+            return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                                  "The locktoken specified in the "
                                  "\"Lock-Token:\" header did not specify one "
                                  "of this resource's locktoken(s).");
         }
 
         if (reason == NULL) {
-            return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0,
+            return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0, 0,
                                  "The preconditions specified by the \"If:\" "
                                  "header did not match this resource.");
         }
 
-        return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0,
+        return dav_new_error(p, HTTP_PRECONDITION_FAILED, 0, 0,
                              apr_psprintf(p,
                                          "The precondition(s) specified by "
                                          "the \"If:\" header did not match "
@@ -1377,13 +1363,13 @@ static dav_error * dav_validate_resource_state(apr_pool_t *p,
     ** We want to note the 400 (Bad Request) in favor of a 423 (Locked).
     */
     if (if_header->dummy_header) {
-        return dav_new_error(p, HTTP_BAD_REQUEST, 0,
+        return dav_new_error(p, HTTP_BAD_REQUEST, 0, 0,
                              "The locktoken specified in the "
                              "\"Lock-Token:\" header did not specify one "
                              "of this resource's locktoken(s).");
     }
 
-    return dav_new_error(p, HTTP_LOCKED, 1 /* error_id */,
+    return dav_new_error(p, HTTP_LOCKED, 1 /* error_id */, 0,
                          "This resource is locked and the \"If:\" header "
                          "did not specify one of the "
                          "locktokens for this resource's lock(s).");
@@ -1435,11 +1421,11 @@ static int dav_meets_conditions(request_rec *r, int resource_state)
 
     retVal = ap_meets_conditions(r);
 
-    /* If-None-Match '*' fix. If-None-Match '*' request should succeed 
+    /* If-None-Match '*' fix. If-None-Match '*' request should succeed
      * if the resource does not exist. */
     if (retVal == HTTP_PRECONDITION_FAILED) {
         /* Note. If if_none_match != NULL, if_none_match is the culprit.
-         * Since, in presence of If-None-Match, 
+         * Since, in presence of If-None-Match,
          * other If-* headers are undefined. */
         if ((if_none_match =
             apr_table_get(r->headers_in, "If-None-Match")) != NULL) {
@@ -1492,7 +1478,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         ** ### bleck. we can't return errors for other URIs unless we have
         ** ### a "response" ptr.
         */
-        return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0,
+        return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0, 0,
                              "DESIGN ERROR: dav_validate_request called "
                              "with depth>0, but no response ptr.");
     }
@@ -1524,7 +1510,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         apr_table_unset(r->headers_out, "ETag");
     }
     if (result != OK) {
-        return dav_new_error(r->pool, result, 0, NULL);
+        return dav_new_error(r->pool, result, 0, 0, NULL);
     }
 
     /* always parse (and later process) the If: header */
@@ -1570,10 +1556,8 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         }
     }
 
-    /* (1) Validate the specified resource, at the specified depth.
-     * Avoid the walk there is no if_header and we aren't planning
-     * to modify this resource. */
-    if (resource->exists && depth > 0 && !(!if_header && flags & DAV_VALIDATE_NO_MODIFY)) {
+    /* (1) Validate the specified resource, at the specified depth */
+    if (resource->exists && depth > 0) {
         dav_walker_ctx ctx = { { 0 } };
         dav_response *multi_status;
 
@@ -1610,7 +1594,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         err = (*repos_hooks->get_parent_resource)(resource, &parent_resource);
 
         if (err == NULL && parent_resource == NULL) {
-            err = dav_new_error(r->pool, HTTP_FORBIDDEN, 0,
+            err = dav_new_error(r->pool, HTTP_FORBIDDEN, 0, 0,
                                 "Cannot access parent of repository root.");
         }
         else if (err == NULL) {
@@ -1666,7 +1650,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
 
         if ((flags & DAV_VALIDATE_USE_424) != 0) {
             /* manufacture a 424 error to hold the multistatus response(s) */
-            return dav_new_error(r->pool, HTTP_FAILED_DEPENDENCY, 0,
+            return dav_new_error(r->pool, HTTP_FAILED_DEPENDENCY, 0, 0,
                                  "An error occurred on another resource, "
                                  "preventing the requested operation on "
                                  "this resource.");
@@ -1701,7 +1685,7 @@ DAV_DECLARE(dav_error *) dav_validate_request(request_rec *r,
         *response = new_response;
 
         /* manufacture a 207 error for the multistatus response(s) */
-        return dav_new_error(r->pool, HTTP_MULTI_STATUS, 0,
+        return dav_new_error(r->pool, HTTP_MULTI_STATUS, 0, 0,
                              "Error(s) occurred on resources during the "
                              "validation process.");
     }
@@ -1745,7 +1729,7 @@ DAV_DECLARE(dav_error *) dav_get_locktoken_list(request_rec *r,
     }
     if (*ltl == NULL) {
         /* No nodes added */
-        return dav_new_error(r->pool, HTTP_BAD_REQUEST, DAV_ERR_IF_ABSENT,
+        return dav_new_error(r->pool, HTTP_BAD_REQUEST, DAV_ERR_IF_ABSENT, 0,
                              "No locktokens were specified in the \"If:\" "
                              "header, so the refresh could not be performed.");
     }
@@ -1842,7 +1826,7 @@ static dav_error * dav_can_auto_checkout(
             const dav_hooks_locks *locks_hooks = DAV_GET_HOOKS_LOCKS(r);
 
             if (locks_hooks == NULL) {
-                return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0,
+                return dav_new_error(r->pool, HTTP_INTERNAL_SERVER_ERROR, 0, 0,
                                      "Auto-checkout is only enabled for locked resources, "
                                      "but there is no lock provider.");
             }
@@ -1897,7 +1881,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
             goto done;
 
         if (parent == NULL || !parent->exists) {
-            err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+            err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                 apr_psprintf(r->pool,
                                             "Missing one or more intermediate "
                                             "collections. Cannot create resource %s.",
@@ -1919,7 +1903,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
             }
 
             if (!checkout_parent) {
-                err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+                err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                     "<DAV:cannot-modify-checked-in-parent>");
                 goto done;
             }
@@ -1978,7 +1962,7 @@ DAV_DECLARE(dav_error *) dav_auto_checkout(
         }
 
         if (!checkout_resource) {
-            err = dav_new_error(r->pool, HTTP_CONFLICT, 0,
+            err = dav_new_error(r->pool, HTTP_CONFLICT, 0, 0,
                                 "<DAV:cannot-modify-version-controlled-content>");
             goto done;
         }
