@@ -99,7 +99,7 @@ static void *merge_alias_dir_config(apr_pool_t *p, void *basev, void *overridesv
 static int alias_matches(const char *uri, const char *alias_fakename);
 
 static const char *add_alias_internal(cmd_parms *cmd, void *dummy,
-                                      const char *f, const char *r,
+                                      const char *fake, const char *real,
                                       int use_regex)
 {
     server_rec *s = cmd->server;
@@ -109,13 +109,13 @@ static const char *add_alias_internal(cmd_parms *cmd, void *dummy,
     alias_entry *entries = (alias_entry *)conf->aliases->elts;
     int i;
 
-    /* XX r can NOT be relative to DocumentRoot here... compat bug. */
+    /* XXX: real can NOT be relative to DocumentRoot here... compat bug. */
 
     if (use_regex) {
-        new->regexp = ap_pregcomp(cmd->pool, f, AP_REG_EXTENDED);
+        new->regexp = ap_pregcomp(cmd->pool, fake, AP_REG_EXTENDED);
         if (new->regexp == NULL)
             return "Regular expression could not be compiled.";
-        new->real = r;
+        new->real = real;
     }
     else {
         /* XXX This may be optimized, but we must know that new->real
@@ -123,9 +123,9 @@ static const char *add_alias_internal(cmd_parms *cmd, void *dummy,
          * and just canonicalizing the remainder.  Not till I finish
          * cleaning out the old ap_canonical stuff first.
          */
-        new->real = r;
+        new->real = real;
     }
-    new->fake = f;
+    new->fake = fake;
     new->handler = cmd->info;
 
     /* check for overlapping (Script)Alias directives
@@ -133,18 +133,18 @@ static const char *add_alias_internal(cmd_parms *cmd, void *dummy,
      */
     if (!use_regex) {
         for (i = 0; i < conf->aliases->nelts - 1; ++i) {
-            alias_entry *p = &entries[i];
+            alias_entry *alias = &entries[i];
 
-            if (  (!p->regexp &&  alias_matches(f, p->fake) > 0)
-                || (p->regexp && !ap_regexec(p->regexp, f, 0, NULL, 0))) {
-                ap_log_error(APLOG_MARK, APLOG_WARNING, 0, cmd->server,
+            if (  (!alias->regexp &&  alias_matches(fake, alias->fake) > 0)
+                || (alias->regexp && !ap_regexec(alias->regexp, fake, 0, NULL, 0))) {
+                ap_log_error(APLOG_MARK, APLOG_WARNING, 0, cmd->server, APLOGNO(00671)
                              "The %s directive in %s at line %d will probably "
                              "never match because it overlaps an earlier "
                              "%sAlias%s.",
                              cmd->cmd->name, cmd->directive->filename,
                              cmd->directive->line_num,
-                             p->handler ? "Script" : "",
-                             p->regexp ? "Match" : "");
+                             alias->handler ? "Script" : "",
+                             alias->regexp ? "Match" : "");
 
                 break; /* one warning per alias should be sufficient */
             }
@@ -154,16 +154,16 @@ static const char *add_alias_internal(cmd_parms *cmd, void *dummy,
     return NULL;
 }
 
-static const char *add_alias(cmd_parms *cmd, void *dummy, const char *f,
-                             const char *r)
+static const char *add_alias(cmd_parms *cmd, void *dummy, const char *fake,
+                             const char *real)
 {
-    return add_alias_internal(cmd, dummy, f, r, 0);
+    return add_alias_internal(cmd, dummy, fake, real, 0);
 }
 
-static const char *add_alias_regex(cmd_parms *cmd, void *dummy, const char *f,
-                                   const char *r)
+static const char *add_alias_regex(cmd_parms *cmd, void *dummy,
+                                   const char *fake, const char *real)
 {
-    return add_alias_internal(cmd, dummy, f, r, 1);
+    return add_alias_internal(cmd, dummy, fake, real, 1);
 }
 
 static const char *add_redirect_internal(cmd_parms *cmd,
@@ -177,8 +177,8 @@ static const char *add_redirect_internal(cmd_parms *cmd,
                                                          &alias_module);
     int status = (int) (long) cmd->info;
     int grokarg1 = 1;
-    ap_regex_t *r = NULL;
-    const char *f = arg2;
+    ap_regex_t *regex = NULL;
+    const char *fake = arg2;
     const char *url = arg3;
 
     /*
@@ -211,13 +211,13 @@ static const char *add_redirect_internal(cmd_parms *cmd,
      * one, so we don't want to re-arrange
      */
     if (!arg3 && !grokarg1) {
-        f = arg1;
+        fake = arg1;
         url = arg2;
     }
 
     if (use_regex) {
-        r = ap_pregcomp(cmd->pool, f, AP_REG_EXTENDED);
-        if (r == NULL)
+        regex = ap_pregcomp(cmd->pool, fake, AP_REG_EXTENDED);
+        if (regex == NULL)
             return "Regular expression could not be compiled.";
     }
 
@@ -240,9 +240,9 @@ static const char *add_redirect_internal(cmd_parms *cmd,
     else
         new = apr_array_push(serverconf->redirects);
 
-    new->fake = f;
+    new->fake = fake;
     new->real = url;
-    new->regexp = r;
+    new->regexp = regex;
     new->redir_status = status;
     return NULL;
 }
@@ -333,8 +333,11 @@ static int alias_matches(const char *uri, const char *alias_fakename)
     return urip - uri;
 }
 
+static char magic_error_value;
+#define PREGSUB_ERROR      (&magic_error_value)
+
 static char *try_alias_list(request_rec *r, apr_array_header_t *aliases,
-                            int doesc, int *status)
+                            int is_redir, int *status)
 {
     alias_entry *entries = (alias_entry *) aliases->elts;
     ap_regmatch_t regm[AP_MAX_REG_MATCH];
@@ -342,55 +345,75 @@ static char *try_alias_list(request_rec *r, apr_array_header_t *aliases,
     int i;
 
     for (i = 0; i < aliases->nelts; ++i) {
-        alias_entry *p = &entries[i];
+        alias_entry *alias = &entries[i];
         int l;
 
-        if (p->regexp) {
-            if (!ap_regexec(p->regexp, r->uri, AP_MAX_REG_MATCH, regm, 0)) {
-                if (p->real) {
-                    found = ap_pregsub(r->pool, p->real, r->uri,
+        if (alias->regexp) {
+            if (!ap_regexec(alias->regexp, r->uri, AP_MAX_REG_MATCH, regm, 0)) {
+                if (alias->real) {
+                    found = ap_pregsub(r->pool, alias->real, r->uri,
                                        AP_MAX_REG_MATCH, regm);
-                    if (found && doesc) {
-                        apr_uri_t uri;
-                        apr_uri_parse(r->pool, found, &uri);
-                        /* Do not escape the query string or fragment. */
-                        found = apr_uri_unparse(r->pool, &uri,
-                                                APR_URI_UNP_OMITQUERY);
-                        found = ap_escape_uri(r->pool, found);
-                        if (uri.query) {
-                            found = apr_pstrcat(r->pool, found, "?",
-                                                uri.query, NULL);
-                        }
-                        if (uri.fragment) {
-                            found = apr_pstrcat(r->pool, found, "#",
-                                                uri.fragment, NULL);
-                        }
+                    if (found) {
+                       if (is_redir) {
+                            apr_uri_t uri;
+                            apr_uri_parse(r->pool, found, &uri);
+                            /* Do not escape the query string or fragment. */
+                            found = apr_uri_unparse(r->pool, &uri,
+                                                    APR_URI_UNP_OMITQUERY);
+                            found = ap_escape_uri(r->pool, found);
+                            if (uri.query) {
+                                found = apr_pstrcat(r->pool, found, "?",
+                                                    uri.query, NULL);
+                            }
+                            if (uri.fragment) {
+                                found = apr_pstrcat(r->pool, found, "#",
+                                                    uri.fragment, NULL);
+                            }
+                       }
+                       else {
+                           int pathlen = strlen(found) -
+                                         (strlen(r->uri + regm[0].rm_eo));
+                           AP_DEBUG_ASSERT(pathlen >= 0);
+                           AP_DEBUG_ASSERT(pathlen <= strlen(found));
+                           ap_set_context_info(r,
+                                               apr_pstrmemdup(r->pool, r->uri,
+                                                              regm[0].rm_eo),
+                                               apr_pstrmemdup(r->pool, found,
+                                                              pathlen));
+                       }
+                    }
+                    else {
+                        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00672)
+                                      "Regex substitution in '%s' failed. "
+                                      "Replacement too long?", alias->real);
+                        return PREGSUB_ERROR;
                     }
                 }
                 else {
                     /* need something non-null */
-                    found = apr_pstrdup(r->pool, "");
+                    found = "";
                 }
             }
         }
         else {
-            l = alias_matches(r->uri, p->fake);
+            l = alias_matches(r->uri, alias->fake);
 
             if (l > 0) {
-                if (doesc) {
+                ap_set_context_info(r, alias->fake, alias->real);
+                if (is_redir) {
                     char *escurl;
                     escurl = ap_os_escape_path(r->pool, r->uri + l, 1);
 
-                    found = apr_pstrcat(r->pool, p->real, escurl, NULL);
+                    found = apr_pstrcat(r->pool, alias->real, escurl, NULL);
                 }
                 else
-                    found = apr_pstrcat(r->pool, p->real, r->uri + l, NULL);
+                    found = apr_pstrcat(r->pool, alias->real, r->uri + l, NULL);
             }
         }
 
         if (found) {
-            if (p->handler) {    /* Set handler, and leave a note for mod_cgi */
-                r->handler = p->handler;
+            if (alias->handler) {    /* Set handler, and leave a note for mod_cgi */
+                r->handler = alias->handler;
                 apr_table_setn(r->notes, "alias-forced-type", r->handler);
             }
             /* XXX This is as SLOW as can be, next step, we optimize
@@ -398,11 +421,11 @@ static char *try_alias_list(request_rec *r, apr_array_header_t *aliases,
              * canonicalized.  After I finish eliminating os canonical.
              * Better fail test for ap_server_root_relative needed here.
              */
-            if (!doesc) {
+            if (!is_redir) {
                 found = ap_server_root_relative(r->pool, found);
             }
             if (found) {
-                *status = p->redir_status;
+                *status = alias->redir_status;
             }
             return found;
         }
@@ -424,33 +447,34 @@ static int translate_alias_redir(request_rec *r)
     }
 
     if ((ret = try_alias_list(r, serverconf->redirects, 1, &status)) != NULL) {
+        if (ret == PREGSUB_ERROR)
+            return HTTP_INTERNAL_SERVER_ERROR;
         if (ap_is_HTTP_REDIRECT(status)) {
-            char *orig_target = ret;
             if (ret[0] == '/') {
+                char *orig_target = ret;
 
                 ret = ap_construct_url(r->pool, ret, r);
-                ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(00673)
                               "incomplete redirection target of '%s' for "
                               "URI '%s' modified to '%s'",
                               orig_target, r->uri, ret);
             }
             if (!ap_is_url(ret)) {
-                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                status = HTTP_INTERNAL_SERVER_ERROR;
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00674)
                               "cannot redirect '%s' to '%s'; "
                               "target is not a valid absoluteURI or abs_path",
                               r->uri, ret);
-                /* restore the config value, so as not to get a
-                 * "regression" on existing "working" configs.
+            }
+            else {
+                /* append requested query only, if the config didn't
+                 * supply its own.
                  */
-                ret = orig_target;
+                if (r->args && !ap_strchr(ret, '?')) {
+                    ret = apr_pstrcat(r->pool, ret, "?", r->args, NULL);
+                }
+                apr_table_setn(r->headers_out, "Location", ret);
             }
-            /* append requested query only, if the config didn't
-             * supply its own.
-             */
-            if (r->args && !ap_strchr(ret, '?')) {
-                ret = apr_pstrcat(r->pool, ret, "?", r->args, NULL);
-            }
-            apr_table_setn(r->headers_out, "Location", ret);
         }
         return status;
     }
@@ -474,19 +498,21 @@ static int fixup_redir(request_rec *r)
     /* It may have changed since last time, so try again */
 
     if ((ret = try_alias_list(r, dirconf->redirects, 1, &status)) != NULL) {
+        if (ret == PREGSUB_ERROR)
+            return HTTP_INTERNAL_SERVER_ERROR;
         if (ap_is_HTTP_REDIRECT(status)) {
             if (ret[0] == '/') {
                 char *orig_target = ret;
 
                 ret = ap_construct_url(r->pool, ret, r);
-                ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r,
+                ap_log_rerror(APLOG_MARK, APLOG_DEBUG, 0, r, APLOGNO(00675)
                               "incomplete redirection target of '%s' for "
                               "URI '%s' modified to '%s'",
                               orig_target, r->uri, ret);
             }
             if (!ap_is_url(ret)) {
                 status = HTTP_INTERNAL_SERVER_ERROR;
-                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00676)
                               "cannot redirect '%s' to '%s'; "
                               "target is not a valid absoluteURI or abs_path",
                               r->uri, ret);
@@ -516,7 +542,7 @@ static void register_hooks(apr_pool_t *p)
     ap_hook_fixups(fixup_redir,NULL,NULL,APR_HOOK_MIDDLE);
 }
 
-module AP_MODULE_DECLARE_DATA alias_module =
+AP_DECLARE_MODULE(alias) =
 {
     STANDARD20_MODULE_STUFF,
     create_alias_dir_config,       /* dir config creater */
