@@ -17,6 +17,8 @@
 #include "ajp_header.h"
 #include "ajp.h"
 
+APLOG_USE_MODULE(proxy_ajp);
+
 static const char *response_trans_headers[] = {
     "Content-Type",
     "Content-Language",
@@ -137,7 +139,7 @@ static const unsigned char sc_for_req_method_table[] = {
     SC_M_PUT,
     SC_M_POST,
     SC_M_DELETE,
-    0,                      /* M_DELETE */
+    0,                      /* M_CONNECT */
     SC_M_OPTIONS,
     SC_M_TRACE,
     0,                      /* M_PATCH  */
@@ -221,14 +223,13 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     const apr_array_header_t *arr = apr_table_elts(r->subprocess_env);
     const apr_table_entry_t *elts = (const apr_table_entry_t *)arr->elts;
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-                         "Into ajp_marshal_into_msgb");
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE8, 0, r, "Into ajp_marshal_into_msgb");
 
     if ((method = sc_for_req_method_by_id(r)) == UNKNOWN_METHOD) {
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_marshal_into_msgb - Sending unknown method %s as request attribute",
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00967)
+               "ajp_marshal_into_msgb - No such method %s",
                r->method);
-        method = SC_M_JK_STORED;
+        return AJP_EBAD_METHOD;
     }
 
     is_ssl = (apr_byte_t) ap_proxy_conn_is_https(r->connection);
@@ -243,19 +244,19 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     ajp_msg_reset(msg);
 
     if (ajp_msg_append_uint8(msg, CMD_AJP13_FORWARD_REQUEST)     ||
-        ajp_msg_append_uint8(msg, method)                        ||
+        ajp_msg_append_uint8(msg, (apr_byte_t) method)           ||
         ajp_msg_append_string(msg, r->protocol)                  ||
         ajp_msg_append_string(msg, uri->path)                    ||
-        ajp_msg_append_string(msg, r->connection->remote_ip)     ||
+        ajp_msg_append_string(msg, r->useragent_ip)              ||
         ajp_msg_append_string(msg, remote_host)                  ||
         ajp_msg_append_string(msg, ap_get_server_name(r))        ||
         ajp_msg_append_uint16(msg, (apr_uint16_t)r->connection->local_addr->port) ||
         ajp_msg_append_uint8(msg, is_ssl)                        ||
         ajp_msg_append_uint16(msg, (apr_uint16_t) num_headers)) {
 
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00968)
                "ajp_marshal_into_msgb: "
-               "Error appending the message begining");
+               "Error appending the message beginning");
         return APR_EGENERAL;
     }
 
@@ -266,7 +267,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
 
         if ((sc = sc_for_req_header(elts[i].key)) != UNKNOWN_METHOD) {
             if (ajp_msg_append_uint16(msg, (apr_uint16_t)sc)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00969)
                        "ajp_marshal_into_msgb: "
                        "Error appending the header name");
                 return AJP_EOVERFLOW;
@@ -274,7 +275,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
         }
         else {
             if (ajp_msg_append_string(msg, elts[i].key)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00970)
                        "ajp_marshal_into_msgb: "
                        "Error appending the header name");
                 return AJP_EOVERFLOW;
@@ -282,12 +283,12 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
         }
 
         if (ajp_msg_append_string(msg, elts[i].val)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00971)
                    "ajp_marshal_into_msgb: "
                    "Error appending the header value");
             return AJP_EOVERFLOW;
         }
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r,
                    "ajp_marshal_into_msgb: Header[%d] [%s] = [%s]",
                    i, elts[i].key, elts[i].val);
     }
@@ -296,7 +297,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     if (s->secret) {
         if (ajp_msg_append_uint8(msg, SC_A_SECRET) ||
             ajp_msg_append_string(msg, s->secret)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r,
                    "Error ajp_marshal_into_msgb - "
                    "Error appending secret");
             return APR_EGENERAL;
@@ -307,7 +308,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     if (r->user) {
         if (ajp_msg_append_uint8(msg, SC_A_REMOTE_USER) ||
             ajp_msg_append_string(msg, r->user)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00972)
                    "ajp_marshal_into_msgb: "
                    "Error appending the remote user");
             return AJP_EOVERFLOW;
@@ -316,7 +317,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     if (r->ap_auth_type) {
         if (ajp_msg_append_uint8(msg, SC_A_AUTH_TYPE) ||
             ajp_msg_append_string(msg, r->ap_auth_type)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00973)
                    "ajp_marshal_into_msgb: "
                    "Error appending the auth type");
             return AJP_EOVERFLOW;
@@ -326,7 +327,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     if (uri->query) {
         if (ajp_msg_append_uint8(msg, SC_A_QUERY_STRING) ||
             ajp_msg_append_string(msg, uri->query)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00974)
                    "ajp_marshal_into_msgb: "
                    "Error appending the query string");
             return AJP_EOVERFLOW;
@@ -335,7 +336,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     if ((session_route = apr_table_get(r->notes, "session-route"))) {
         if (ajp_msg_append_uint8(msg, SC_A_JVM_ROUTE) ||
             ajp_msg_append_string(msg, session_route)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00975)
                    "ajp_marshal_into_msgb: "
                    "Error appending the jvm route");
             return AJP_EOVERFLOW;
@@ -358,9 +359,9 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
             && envvar[0]) {
             if (ajp_msg_append_uint8(msg, SC_A_SSL_CERT)
                 || ajp_msg_append_string(msg, envvar)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                             "ajp_marshal_into_msgb: "
-                             "Error appending the SSL certificates");
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00976)
+                              "ajp_marshal_into_msgb: "
+                              "Error appending the SSL certificates");
                 return AJP_EOVERFLOW;
             }
         }
@@ -370,9 +371,9 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
             && envvar[0]) {
             if (ajp_msg_append_uint8(msg, SC_A_SSL_CIPHER)
                 || ajp_msg_append_string(msg, envvar)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                             "ajp_marshal_into_msgb: "
-                             "Error appending the SSL ciphers");
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00977)
+                              "ajp_marshal_into_msgb: "
+                              "Error appending the SSL ciphers");
                 return AJP_EOVERFLOW;
             }
         }
@@ -382,9 +383,9 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
             && envvar[0]) {
             if (ajp_msg_append_uint8(msg, SC_A_SSL_SESSION)
                 || ajp_msg_append_string(msg, envvar)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                             "ajp_marshal_into_msgb: "
-                             "Error appending the SSL session");
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00978)
+                              "ajp_marshal_into_msgb: "
+                              "Error appending the SSL session");
                 return AJP_EOVERFLOW;
             }
         }
@@ -396,24 +397,13 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
 
             if (ajp_msg_append_uint8(msg, SC_A_SSL_KEY_SIZE)
                 || ajp_msg_append_uint16(msg, (unsigned short) atoi(envvar))) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                             "Error ajp_marshal_into_msgb - "
-                             "Error appending the SSL key size");
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00979)
+                              "ajp_marshal_into_msgb: "
+                              "Error appending the SSL key size");
                 return APR_EGENERAL;
             }
         }
     }
-    /* If the method was unrecognized, encode it as an attribute */
-    if (method == SC_M_JK_STORED) {
-        if (ajp_msg_append_uint8(msg, SC_A_STORED_METHOD)
-            || ajp_msg_append_string(msg, r->method)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                         "ajp_marshal_into_msgb: "
-                         "Error appending the method '%s' as request attribute",
-                         r->method);
-            return AJP_EOVERFLOW;
-        }
-    }
     /* Forward the remote port information, which was forgotten
      * from the builtin data of the AJP 13 protocol.
      * Since the servlet spec allows to retrieve it via getRemotePort(),
@@ -423,31 +413,11 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
      */
     {
         const char *key = SC_A_REQ_REMOTE_PORT;
-        char *val = apr_itoa(r->pool, r->connection->remote_addr->port);
+        char *val = apr_itoa(r->pool, r->useragent_addr->port);
         if (ajp_msg_append_uint8(msg, SC_A_REQ_ATTRIBUTE) ||
             ajp_msg_append_string(msg, key)   ||
             ajp_msg_append_string(msg, val)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-                    "ajp_marshal_into_msgb: "
-                    "Error appending attribute %s=%s",
-                    key, val);
-            return AJP_EOVERFLOW;
-        }
-    }
-    /* Forward the local ip address information, which was forgotten
-     * from the builtin data of the AJP 13 protocol.
-     * Since the servlet spec allows to retrieve it via getLocalAddr(),
-     * we provide the address to the Tomcat connector as a request
-     * attribute. Modern Tomcat versions know how to retrieve
-     * the local address from this attribute.
-     */
-    {
-        const char *key = SC_A_REQ_LOCAL_ADDR;
-        char *val = r->connection->local_ip;
-        if (ajp_msg_append_uint8(msg, SC_A_REQ_ATTRIBUTE) ||
-            ajp_msg_append_string(msg, key)   ||
-            ajp_msg_append_string(msg, val)) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00980)
                     "ajp_marshal_into_msgb: "
                     "Error appending attribute %s=%s",
                     key, val);
@@ -462,7 +432,7 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
             if (ajp_msg_append_uint8(msg, SC_A_REQ_ATTRIBUTE) ||
                 ajp_msg_append_string(msg, elts[i].key + 4)   ||
                 ajp_msg_append_string(msg, elts[i].val)) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00981)
                         "ajp_marshal_into_msgb: "
                         "Error appending attribute %s=%s",
                         elts[i].key, elts[i].val);
@@ -472,13 +442,13 @@ static apr_status_t ajp_marshal_into_msgb(ajp_msg_t *msg,
     }
 
     if (ajp_msg_append_uint8(msg, SC_A_ARE_DONE)) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00982)
                "ajp_marshal_into_msgb: "
                "Error appending the message end");
         return AJP_EOVERFLOW;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE8, 0, r,
             "ajp_marshal_into_msgb: Done");
     return APR_SUCCESS;
 }
@@ -527,7 +497,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
     rc = ajp_msg_get_uint16(msg, &status);
 
     if (rc != APR_SUCCESS) {
-         ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+         ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00983)
                 "ajp_unmarshal_response: Null status");
         return rc;
     }
@@ -535,7 +505,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
 
     rc = ajp_msg_get_string(msg, &ptr);
     if (rc == APR_SUCCESS) {
-#if defined(AS400) || defined(_OSD_POSIX) /* EBCDIC platforms */
+#if APR_CHARSET_EBCDIC /* copy only if we have to */
         ptr = apr_pstrdup(r->pool, ptr);
         ap_xlate_proto_from_ascii(ptr, strlen(ptr));
 #endif
@@ -544,7 +514,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         r->status_line = NULL;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE4, 0, r,
            "ajp_unmarshal_response: status = %d", status);
 
     rc = ajp_msg_get_uint16(msg, &num_headers);
@@ -565,7 +535,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         num_headers = 0;
     }
 
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE4, 0, r,
            "ajp_unmarshal_response: Number of headers is = %d",
            num_headers);
 
@@ -582,7 +552,7 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
             ajp_msg_get_uint16(msg, &name);
             stringname = long_res_header_for_sc(name);
             if (stringname == NULL) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00984)
                        "ajp_unmarshal_response: "
                        "No such sc (%08x)",
                        name);
@@ -592,19 +562,17 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
             name = 0;
             rc = ajp_msg_get_string(msg, &stringname);
             if (rc != APR_SUCCESS) {
-                ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+                ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00985)
                        "ajp_unmarshal_response: "
                        "Null header name");
                 return rc;
             }
-#if defined(AS400) || defined(_OSD_POSIX)
             ap_xlate_proto_from_ascii(stringname, strlen(stringname));
-#endif
         }
 
         rc = ajp_msg_get_string(msg, &value);
         if (rc != APR_SUCCESS) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00986)
                    "ajp_unmarshal_response: "
                    "Null header value");
             return rc;
@@ -624,10 +592,8 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
           value = ap_proxy_location_reverse_map(r, dconf, value);
         }
 
-#if defined(AS400) || defined(_OSD_POSIX)
         ap_xlate_proto_from_ascii(value, strlen(value));
-#endif
-        ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r,
                "ajp_unmarshal_response: Header[%d] [%s] = [%s]",
                        i, stringname, value);
 
@@ -637,8 +603,8 @@ static apr_status_t ajp_unmarshal_response(ajp_msg_t *msg,
         if (strcasecmp(stringname, "Content-Type") == 0) {
              /* add corresponding filter */
             ap_set_content_type(r, apr_pstrdup(r->pool, value));
-            ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_unmarshal_response: ap_set_content_type done");
+            ap_log_rerror(APLOG_MARK, APLOG_TRACE5, 0, r,
+               "ajp_unmarshal_response: ap_set_content_type to '%s'", value);
         }
     }
 
@@ -658,21 +624,22 @@ apr_status_t ajp_send_header(apr_socket_t *sock,
 
     rc = ajp_msg_create(r->pool, buffsize, &msg);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00987)
                "ajp_send_header: ajp_msg_create failed");
         return rc;
     }
 
     rc = ajp_marshal_into_msgb(msg, r, uri);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00988)
                "ajp_send_header: ajp_marshal_into_msgb failed");
         return rc;
     }
 
     rc = ajp_ilink_send(sock, msg);
+    ajp_msg_log(r, msg, "ajp_send_header: ajp_ilink_send packet dump");
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00989)
                "ajp_send_header: ajp_ilink_send failed");
         return rc;
     }
@@ -694,7 +661,7 @@ apr_status_t ajp_read_header(apr_socket_t *sock,
     if (*msg) {
         rc = ajp_msg_reuse(*msg);
         if (rc != APR_SUCCESS) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00990)
                    "ajp_read_header: ajp_msg_reuse failed");
             return rc;
         }
@@ -702,7 +669,7 @@ apr_status_t ajp_read_header(apr_socket_t *sock,
     else {
         rc = ajp_msg_create(r->pool, buffsize, msg);
         if (rc != APR_SUCCESS) {
-            ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+            ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00991)
                    "ajp_read_header: ajp_msg_create failed");
             return rc;
         }
@@ -710,13 +677,20 @@ apr_status_t ajp_read_header(apr_socket_t *sock,
     ajp_msg_reset(*msg);
     rc = ajp_ilink_receive(sock, *msg);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00992)
                "ajp_read_header: ajp_ilink_receive failed");
         return rc;
     }
+    ajp_msg_log(r, *msg, "ajp_read_header: ajp_ilink_receive packet dump");
     rc = ajp_msg_peek_uint8(*msg, &result);
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_read_header: ajp_ilink_received %02x", result);
+    if (rc != APR_SUCCESS) {
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00993)
+                      "ajp_read_header: ajp_msg_peek_uint8 failed");
+        return rc;
+    }
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE1, 0, r,
+               "ajp_read_header: ajp_ilink_received %s (0x%02x)",
+               ajp_type_str(result), result);
     return APR_SUCCESS;
 }
 
@@ -725,8 +699,9 @@ int ajp_parse_type(request_rec  *r, ajp_msg_t *msg)
 {
     apr_byte_t result;
     ajp_msg_peek_uint8(msg, &result);
-    ap_log_error(APLOG_MARK, APLOG_DEBUG, 0, r->server,
-               "ajp_parse_type: got %02x", result);
+    ap_log_rerror(APLOG_MARK, APLOG_TRACE6, 0, r,
+               "ajp_parse_type: got %s (0x%02x)",
+               ajp_type_str(result), result);
     return (int) result;
 }
 
@@ -739,14 +714,15 @@ apr_status_t ajp_parse_header(request_rec  *r, proxy_dir_conf *conf,
 
     rc = ajp_msg_get_uint8(msg, &result);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00994)
                "ajp_parse_headers: ajp_msg_get_byte failed");
         return rc;
     }
     if (result != CMD_AJP13_SEND_HEADERS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-               "ajp_parse_headers: wrong type 0x%02x expecting 0x%02x",
-               result, CMD_AJP13_SEND_HEADERS);
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00995)
+               "ajp_parse_headers: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_SEND_HEADERS), CMD_AJP13_SEND_HEADERS);
         return AJP_EBAD_HEADER;
     }
     return ajp_unmarshal_response(msg, r, conf);
@@ -762,14 +738,15 @@ apr_status_t  ajp_parse_data(request_rec  *r, ajp_msg_t *msg,
 
     rc = ajp_msg_get_uint8(msg, &result);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00996)
                "ajp_parse_data: ajp_msg_get_byte failed");
         return rc;
     }
     if (result != CMD_AJP13_SEND_BODY_CHUNK) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-               "ajp_parse_data: wrong type 0x%02x expecting 0x%02x",
-               result, CMD_AJP13_SEND_BODY_CHUNK);
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00997)
+               "ajp_parse_data: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_SEND_BODY_CHUNK), CMD_AJP13_SEND_BODY_CHUNK);
         return AJP_EBAD_HEADER;
     }
     rc = ajp_msg_get_uint16(msg, len);
@@ -788,7 +765,7 @@ apr_status_t  ajp_parse_data(request_rec  *r, ajp_msg_t *msg,
      */
     expected_len = msg->len - (AJP_HEADER_LEN + AJP_HEADER_SZ_LEN + 1 + 1);
     if (*len != expected_len) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00998)
                "ajp_parse_data: Wrong chunk length. Length of chunk is %i,"
                " expected length is %i.", *len, expected_len);
         return AJP_EBAD_HEADER;
@@ -806,14 +783,15 @@ apr_status_t ajp_parse_reuse(request_rec *r, ajp_msg_t *msg,
 
     rc = ajp_msg_get_uint8(msg, &result);
     if (rc != APR_SUCCESS) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(00999)
                "ajp_parse_reuse: ajp_msg_get_byte failed");
         return rc;
     }
     if (result != CMD_AJP13_END_RESPONSE) {
-        ap_log_error(APLOG_MARK, APLOG_ERR, 0, r->server,
-               "ajp_parse_reuse: wrong type 0x%02x expecting 0x%02x",
-               result, CMD_AJP13_END_RESPONSE);
+        ap_log_rerror(APLOG_MARK, APLOG_ERR, 0, r, APLOGNO(01000)
+               "ajp_parse_reuse: wrong type %s (0x%02x) expecting %s (0x%02x)",
+               ajp_type_str(result), result,
+               ajp_type_str(CMD_AJP13_END_RESPONSE), CMD_AJP13_END_RESPONSE);
         return AJP_EBAD_HEADER;
     }
     return ajp_msg_get_uint8(msg, reuse);
