@@ -1,9 +1,9 @@
-/* Copyright 2001-2005 The Apache Software Foundation or its licensors, as
- * applicable.
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
+/* Licensed to the Apache Software Foundation (ASF) under one or more
+ * contributor license agreements.  See the NOTICE file distributed with
+ * this work for additional information regarding copyright ownership.
+ * The ASF licenses this file to You under the Apache License, Version 2.0
+ * (the "License"); you may not use this file except in compliance with
+ * the License.  You may obtain a copy of the License at
  *
  *     http://www.apache.org/licenses/LICENSE-2.0
  *
@@ -38,12 +38,12 @@
 /* hint for MSL C++ that we're on NetWare platform */
 #define __NETWARE__
 
-/* the FAR keyword has no meaning in a 32-bit environment 
+/* the FAR keyword has no meaning in a 32-bit environment
    but is used in the SDK headers so we take it out */
 #define FAR
 #define far
 
-/* no-op for Codewarrior C compiler; a functions are cdecl 
+/* no-op for Codewarrior C compiler; a functions are cdecl
    by default */
 #define cdecl
 
@@ -63,12 +63,6 @@
 #define __int64 long long
 #endif
 
-/* Don't use the DBM rewrite map for mod_rewrite */
-#define NO_DBM_REWRITEMAP
-
-/* Allow MOD_AUTH_DBM to use APR */
-#define AP_AUTH_DBM_USE_APR
-
 /* Restrict the number of nested includes */
 #define AP_MAX_INCLUDE_DEPTH    48
 
